package com.diagbot.facade;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.dto.HisInquirysForJzDTO;
import com.diagbot.entity.InquiryDetail;
import com.diagbot.entity.InquiryInfo;
import com.diagbot.entity.PatientInfo;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.SexTypeEnum;
import com.diagbot.service.impl.InquiryDetailServiceImpl;
import com.diagbot.service.impl.InquiryInfoServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.vo.AddDiagnoseSecondVO;
import com.diagbot.vo.HisInquirysForJzVO;
import com.diagbot.vo.SaveInquiryToHisVO;

/**
 * @Description: 对接层问诊信息业务逻辑
 * @author: rengb
 * @since 2019-3-18
 */
@Component
public class InquiryInfoFacade extends InquiryInfoServiceImpl{
	
	@Autowired
    private InquiryDetailFacade inquiryDetailFacade;
	@Autowired
	private PatientInfoFacade patientInfoFacade;
	@Autowired
	@Qualifier("inquiryDetailServiceImpl")
	private InquiryDetailServiceImpl inquiryDetailServiceImpl;
	
	/**
     * 问诊记录保存
     *
     * @param saveInquiryToHisVO
     * @return
     */
    public Boolean saveInquiryToHis(SaveInquiryToHisVO saveInquiryToHisVO) {
        
        Date now = new Date();

        QueryWrapper<InquiryInfo> inquiryInfoQe = new QueryWrapper<>();
        inquiryInfoQe.eq("is_deleted", IsDeleteEnum.N.getKey());
        inquiryInfoQe.eq("hospital_id", saveInquiryToHisVO.getHospitalId());
        inquiryInfoQe.eq("hospital_dept_id", saveInquiryToHisVO.getHospitalDeptId());
        inquiryInfoQe.eq("doctor_id", saveInquiryToHisVO.getDoctorId());
        inquiryInfoQe.eq("patient_id", saveInquiryToHisVO.getPatientId());
        inquiryInfoQe.eq("inquiry_code", saveInquiryToHisVO.getInquiryCode());
        InquiryInfo inquiryInfo = this.getOne(inquiryInfoQe);

        Long inquiryId = null;
        InquiryInfo inquiryInfoSave = new InquiryInfo();
        inquiryInfoSave.setHospitalId(saveInquiryToHisVO.getHospitalId());
        inquiryInfoSave.setHospitalDeptId(saveInquiryToHisVO.getHospitalDeptId());
        inquiryInfoSave.setDoctorId(saveInquiryToHisVO.getDoctorId());
        inquiryInfoSave.setPatientId(saveInquiryToHisVO.getPatientId());
        inquiryInfoSave.setInquiryCode(saveInquiryToHisVO.getInquiryCode());
        inquiryInfoSave.setRegVisitedState(saveInquiryToHisVO.getRegVisitedState());
        inquiryInfoSave.setType(saveInquiryToHisVO.getType());
        inquiryInfoSave.setDiagnose(saveInquiryToHisVO.getDiagnose());
        if (inquiryInfo == null) {
            inquiryInfoSave.setGmtCreate(now);
            inquiryInfoSave.setGmtModified(now);
            this.save(inquiryInfoSave);
            inquiryId = inquiryInfoSave.getId();
        } else {
            inquiryId = inquiryInfo.getId();
            inquiryInfoSave.setId(inquiryId);
            inquiryInfoSave.setGmtModified(now);
            this.updateById(inquiryInfoSave);
            
            QueryWrapper<InquiryDetail> inquiryDetailQe = new QueryWrapper<>();
            inquiryDetailQe.eq("inquiry_id", inquiryId);
            inquiryDetailQe.ne("type", 9);
            inquiryDetailFacade.remove(inquiryDetailQe);
        }

        List<InquiryDetail> inquiryDetailList = BeanUtil.listCopyTo(saveInquiryToHisVO.getDetailList(), InquiryDetail.class);
        for(InquiryDetail i : inquiryDetailList){
        	i.setInquiryId(inquiryId);
        	i.setGmtCreate(now);
        	i.setGmtModified(now);
        }
        inquiryDetailServiceImpl.saveBatch(inquiryDetailList);

        return true;
    }
    
    /**
     * 急诊-历史病历列表
     * 
     * @param hisInquirysForJzVO
     * @return
     */
    public IPage<HisInquirysForJzDTO> hisInquirysForJz(HisInquirysForJzVO hisInquirysForJzVO){
    	IPage<HisInquirysForJzDTO> ipage = this.baseMapper.hisInquirysForJz(hisInquirysForJzVO);
    	
    	if(ListUtil.isNotEmpty(ipage.getRecords())){
            List<Long> patientIdList = ipage.getRecords().stream().map(i -> i.getPatientId()).distinct().collect(Collectors.toList());
            List<Long> inquiryIdList = ipage.getRecords().stream().map(i -> i.getInquiryId()).distinct().collect(Collectors.toList());

            Map<Long, PatientInfo> patientInfoMap = patientInfoFacade.patientInfoMapByIds(patientIdList);
            Map<Long, List<InquiryDetail>> inquiryDetailMap = inquiryDetailFacade.getDetailsByInquryIds(inquiryIdList);
        	
            ipage.getRecords().forEach(i -> {
                PatientInfo patientInfo = patientInfoMap.get(i.getPatientId());
                if (patientInfo != null) {
                    i.setPatientName(patientInfo.getName());
                    i.setPatientIdNo(patientInfo.getIdNo());
                    i.setPatientSex(SexTypeEnum.getName(patientInfo.getSex()));
                    i.setPatientAge(DateUtil.yearCompare(patientInfo.getBirthday(), DateUtil.now()));
                }
                
                List<InquiryDetail> detailList= inquiryDetailMap.get(i.getInquiryId());
                i.setDetailList(detailList);
                String diagnose = detailList.stream().filter(inquiryDetail->inquiryDetail.getType()==7).map(inquiryDetail->inquiryDetail.getContentValue()).collect(Collectors.toList()).get(0);
                i.setDiagnose(diagnose.split(";")[0]);
            });
    	}
    	
    	return ipage;
    }
    
    
    /**
     * 急诊-添加二次诊断
     * 
     * @param addDiagnoseSecondVO
     * @return
     */
    public Boolean addDiagnoseSecond(AddDiagnoseSecondVO addDiagnoseSecondVO){
    	QueryWrapper<InquiryDetail> inquiryDetailQe = new QueryWrapper<>();
    	inquiryDetailQe.eq("inquiry_id", addDiagnoseSecondVO.getInquiryId());
    	inquiryDetailQe.eq("type", 9);
    	inquiryDetailFacade.remove(inquiryDetailQe);
    	
    	Date now = DateUtil.now();
    	
    	InquiryDetail inquiryDetail = new InquiryDetail();
    	inquiryDetail.setContentValue(addDiagnoseSecondVO.getDiagnoseSecond());
    	inquiryDetail.setType(9);
    	inquiryDetail.setInquiryId(addDiagnoseSecondVO.getInquiryId());
    	inquiryDetail.setGmtCreate(now);
    	inquiryDetail.setGmtModified(now);
    	inquiryDetail.setCreator(addDiagnoseSecondVO.getDoctorId().toString());
    	inquiryDetail.setModifier(addDiagnoseSecondVO.getDoctorId().toString());
    	
    	return inquiryDetailFacade.save(inquiryDetail);
    }
	


}
